==> webkit2gtk_2.21.5.orig.tar.xz: webkitgtk-2.21.5/Source/JavaScriptCore/builtins/ArrayIteratorPrototype.js <==
/*
 * Copyright (C) 2015 Yusuke Suzuki <utatane.tea@gmail.com>.
 * Copyright (C) 2016 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

function next()
{
    "use strict";

    if (this == null)
        @throwTypeError("%ArrayIteratorPrototype%.next requires that |this| not be null or undefined");

    let next = @getByIdDirectPrivate(this, "arrayIteratorNext");
    if (next === @undefined)
        @throwTypeError("%ArrayIteratorPrototype%.next requires that |this| be an Array Iterator instance");

    return next.@call(this);
}

@globalPrivate
function arrayIteratorValueNext()
{
    "use strict";
    var done = true;
    var value;

    var array = @getByIdDirectPrivate(this, "iteratedObject");
    if (!@getByIdDirectPrivate(this, "arrayIteratorIsDone")) {
        var index = @getByIdDirectPrivate(this, "arrayIteratorNextIndex");
        var length = array.length >>> 0;
        if (index >= length)
            @putByIdDirectPrivate(this, "arrayIteratorIsDone", true);
        else {
            @putByIdDirectPrivate(this, "arrayIteratorNextIndex", index + 1);
            done = false;
            value = array[index];
        }
    }

    return { value, done };
}

@globalPrivate
function arrayIteratorKeyNext()
{
    "use strict";
    var done = true;
    var value;

    var array = @getByIdDirectPrivate(this, "iteratedObject");
    if (!@getByIdDirectPrivate(this, "arrayIteratorIsDone")) {
        var index = @getByIdDirectPrivate(this, "arrayIteratorNextIndex");
        var length = array.length >>> 0;
        if (index >= length)
            @putByIdDirectPrivate(this, "arrayIteratorIsDone", true);
        else {
            @putByIdDirectPrivate(this, "arrayIteratorNextIndex", index + 1);
            done = false;
            value = index;
        }
    }

    return { value, done };
}

@globalPrivate
function arrayIteratorKeyValueNext()
{
    "use strict";
    var done = true;
    var value;

    var array = @getByIdDirectPrivate(this, "iteratedObject");
    if (!@getByIdDirectPrivate(this, "arrayIteratorIsDone")) {
        var index = @getByIdDirectPrivate(this, "arrayIteratorNextIndex");
        var length = array.length >>> 0;
        if (index >= length)
            @putByIdDirectPrivate(this, "arrayIteratorIsDone", true);
        else {
            @putByIdDirectPrivate(this, "arrayIteratorNextIndex", index + 1);
            done = false;
            value = [ index, array[index] ];
        }
    }

    return { value, done };
}

==> webkit2gtk_2.27.4.orig.tar.xz: webkitgtk-2.27.4/Source/JavaScriptCore/builtins/ArrayIteratorPrototype.js <==
/*
 * Copyright (C) 2015 Yusuke Suzuki <utatane.tea@gmail.com>.
 * Copyright (C) 2016 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

function next()
{
    "use strict";

    if (!@isArrayIterator(this))
        @throwTypeError("%ArrayIteratorPrototype%.next requires that |this| be an Array Iterator instance");

    var array = @getArrayIteratorInternalField(this, @arrayIteratorFieldIteratedObject);
    var kind = @getArrayIteratorInternalField(this, @arrayIteratorFieldKind);
    return @arrayIteratorNextHelper.@call(this, array, kind);
}

// FIXME: We have to have this because we can't implement this all as one function and meet our inlining heuristics.
// Collectively, next and this have ~130 bytes of bytecodes but our limit is 120.
@globalPrivate
function arrayIteratorNextHelper(array, kind)
{
    "use strict";
    var done = true;
    var value;

    if (!@getArrayIteratorInternalField(this, @arrayIteratorFieldIsDone)) {
        var index = @getArrayIteratorInternalField(this, @arrayIteratorFieldIndex);
        var length = array.length >>> 0;
        if (index < length) {
            @putArrayIteratorInternalField(this, @arrayIteratorFieldIndex, index + 1);
            done = false;
            if (kind === @iterationKindKey)
                value = index;
            else { 
                value = array[index];
                value = kind === @iterationKindValue ? value : [index, value];
            }
        }
    }
    @putArrayIteratorInternalField(this, @arrayIteratorFieldIsDone, done);

    return { value, done };
}
